@@ -79,54 +79,18 @@ pub const POWERS_10: [U256; (MAX_PRECISION * 2 + 1) as usize] = [
     U256::from_u128(337622765642898738890454548373825388544, 293873587705571876),
     U256::from_u128(313686354140541217734174016852339982336, 2938735877055718769),
     U256::from_u128(74322239116966006171368701637485920256, 29387358770557187699),
-    U256::from_u128(
-        62657657327783134786937801511322779648,
-        293873587705571876992,
-    ),
-    U256::from_u128(
-        286294206356892884406003407681459585024,
-        2938735877055718769921,
-    ),
-    U256::from_u128(
-        140683128201421136353037217360450158592,
-        29387358770557187699218,
-    ),
-    U256::from_u128(
-        45701814330457509676873743877428740096,
-        293873587705571876992184,
-    ),
-    U256::from_u128(
-        116735776383636633305362831342519189504,
-        2938735877055718769921841,
-    ),
-    U256::from_u128(
-        146510663073550942663504491129887260672,
-        29387358770557187699218413,
-    ),
-    U256::from_u128(
-        103977163051755572781546481571799760896,
-        293873587705571876992184134,
-    ),
-    U256::from_u128(
-        18924529754740337425340993422692974592,
-        2938735877055718769921841343,
-    ),
-    U256::from_u128(
-        189245297547403374253409934226929745920,
-        29387358770557187699218413430,
-    ),
-    U256::from_u128(
-        191041140869341425217226305110456401920,
-        293873587705571876992184134305,
-    ),
-    U256::from_u128(
-        208999574088721934855390013945722961920,
-        2938735877055718769921841343055,
-    ),
-    U256::from_u128(
-        48301539361588567773652494866620350464,
-        29387358770557187699218413430556,
-    ),
+    U256::from_u128(62657657327783134786937801511322779648, 293873587705571876992),
+    U256::from_u128(286294206356892884406003407681459585024, 2938735877055718769921),
+    U256::from_u128(140683128201421136353037217360450158592, 29387358770557187699218),
+    U256::from_u128(45701814330457509676873743877428740096, 293873587705571876992184),
+    U256::from_u128(116735776383636633305362831342519189504, 2938735877055718769921841),
+    U256::from_u128(146510663073550942663504491129887260672, 29387358770557187699218413),
+    U256::from_u128(103977163051755572781546481571799760896, 293873587705571876992184134),
+    U256::from_u128(18924529754740337425340993422692974592, 2938735877055718769921841343),
+    U256::from_u128(189245297547403374253409934226929745920, 29387358770557187699218413430),
+    U256::from_u128(191041140869341425217226305110456401920, 293873587705571876992184134305),
+    U256::from_u128(208999574088721934855390013945722961920, 2938735877055718769921841343055),
+    U256::from_u128(48301539361588567773652494866620350464, 29387358770557187699218413430556),
     U256::from_u128(
         142733026694947214273150341234435293184,
         293873587705571876992184134305561,
@@ -212,58 +176,19 @@ pub const ROUNDINGS: [U256; (MAX_PRECISION * 2 + 1) as usize] = [
     U256::from_u128(289107031818895707715222413364105838592, 14693679385278593),
     U256::from_u128(168811382821449369445227274186912694272, 146936793852785938),
     U256::from_u128(326984360530739840598774312142054096896, 1469367938527859384),
-    U256::from_u128(
-        207302303018952234817371654534627065856,
-        14693679385278593849,
-    ),
-    U256::from_u128(
-        31328828663891567393468900755661389824,
-        146936793852785938496,
-    ),
-    U256::from_u128(
-        313288286638915673934689007556613898240,
-        1469367938527859384960,
-    ),
-    U256::from_u128(
-        70341564100710568176518608680225079296,
-        14693679385278593849609,
-    ),
-    U256::from_u128(
-        22850907165228754838436871938714370048,
-        146936793852785938496092,
-    ),
-    U256::from_u128(
-        228509071652287548384368719387143700480,
-        1469367938527859384960920,
-    ),
-    U256::from_u128(
-        243396514997244703063439549280827736064,
-        14693679385278593849609206,
-    ),
-    U256::from_u128(
-        51988581525877786390773240785899880448,
-        146936793852785938496092067,
-    ),
-    U256::from_u128(
-        179603448337839400444357800427230593024,
-        1469367938527859384960920671,
-    ),
-    U256::from_u128(
-        94622648773701687126704967113464872960,
-        14693679385278593849609206715,
-    ),
-    U256::from_u128(
-        265661753895139944340300456271112306688,
-        146936793852785938496092067152,
-    ),
-    U256::from_u128(
-        274640970504830199159382310688745586688,
-        1469367938527859384960920671527,
-    ),
-    U256::from_u128(
-        24150769680794283886826247433310175232,
-        14693679385278593849609206715278,
-    ),
+    U256::from_u128(207302303018952234817371654534627065856, 14693679385278593849),
+    U256::from_u128(31328828663891567393468900755661389824, 146936793852785938496),
+    U256::from_u128(313288286638915673934689007556613898240, 1469367938527859384960),
+    U256::from_u128(70341564100710568176518608680225079296, 14693679385278593849609),
+    U256::from_u128(22850907165228754838436871938714370048, 146936793852785938496092),
+    U256::from_u128(228509071652287548384368719387143700480, 1469367938527859384960920),
+    U256::from_u128(243396514997244703063439549280827736064, 14693679385278593849609206),
+    U256::from_u128(51988581525877786390773240785899880448, 146936793852785938496092067),
+    U256::from_u128(179603448337839400444357800427230593024, 1469367938527859384960920671),
+    U256::from_u128(94622648773701687126704967113464872960, 14693679385278593849609206715),
+    U256::from_u128(265661753895139944340300456271112306688, 146936793852785938496092067152),
+    U256::from_u128(274640970504830199159382310688745586688, 1469367938527859384960920671527),
+    U256::from_u128(24150769680794283886826247433310175232, 14693679385278593849609206715278),
     U256::from_u128(
         241507696807942838868262474333101752320,
         146936793852785938496092067152780,
@@ -337,20 +262,18 @@ impl U256 {
     }
 
     #[inline(always)]
-    pub fn overflowing_add<T: Into<U256>>(&self, other: T) -> (U256, bool) {
-        let (add, overflow) = self.0.overflowing_add(other.into().0);
-        (U256(add), overflow)
+    pub fn checked_add<T: Into<U256>>(&self, other: T) -> Option<U256> {
+        self.0.checked_add(other.into().0).map(U256)
     }
 
     #[inline(always)]
-    pub fn overflowing_sub<T: Into<U256>>(&self, other: T) -> (U256, bool) {
-        let (add, overflow) = self.0.overflowing_sub(other.into().0);
-        (U256(add), overflow)
+    pub fn checked_sub<T: Into<U256>>(&self, other: T) -> Option<U256> {
+        self.0.checked_sub(other.into().0).map(U256)
     }
 
     #[inline(always)]
-    pub fn checked_mul(&self, other: U256) -> Option<U256> {
-        self.0.checked_mul(other.0).map(U256)
+    pub fn checked_mul<T: Into<U256>>(&self, other: T) -> Option<U256> {
+        self.0.checked_mul(other.into().0).map(U256)
     }
 
     #[inline(always)]
@@ -594,18 +517,10 @@ mod tests {
         assert(U256::from(1_0000_0000_0001u128), 13);
         assert(U256::from(1_0000_0000_0000_0000u128), 17);
         assert(U256::from(1_0000_0000_0000_0000_0000_0000_0000u128), 29);
+        assert(U256::from(10_0000_0000_0000_0000_0000_0000_0000_0000_0000u128), 38);
+        assert(U256::from(100_0000_0000_0000_0000_0000_0000_0000_0000_0000u128), 39);
         assert(
-            U256::from(10_0000_0000_0000_0000_0000_0000_0000_0000_0000u128),
-            38,
-        );
-        assert(
-            U256::from(100_0000_0000_0000_0000_0000_0000_0000_0000_0000u128),
-            39,
-        );
-        assert(
-            U256(
-                ethnum::U256::from_str("10000000000000000000000000000000000000000000000").unwrap(),
-            ),
+            U256(ethnum::U256::from_str("10000000000000000000000000000000000000000000000").unwrap()),
             47,
         );
     }
